package nckd.jxccl.base.org.helper;

import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import nckd.jxccl.base.orm.helper.QFilterCommonHelper;
import nckd.jxccl.base.orm.helper.QFilterPersonHelper;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 人员辅助类
 * @author W.Y.C
 * @date 2025/7/7 18:23
 * @version 1.0
 */
public class PersonHelper {

    /** 组织人 */
    public static final String DEP_EMP_ENTITY_ID = "hrpi_depemp";

    /**
     * 查询根据条件查询有效组织人（任职类型为全职、用工关系状态为在职、组织团队分类为行政组织、业务状态为生效中、最新版本、初始状态为完成）
     * @param selectFields 查询字段
     * @param filters 自定义过滤条件
     * @return: kd.bos.dataentity.entity.DynamicObjectCollection
     * @author W.Y.C
     * @date: 2025/07/07 18:37
     */
    public static DynamicObjectCollection getDepEmpEffectivePerson(String selectFields, QFilter... filters) {
        // 构建基础过滤条件
        QFilter baseFilter = QFilterCommonHelper.getBusinessStatusFilter()
                .and(QFilterPersonHelper.getPostCategoryOfStaffFilter())
                .and(QFilterPersonHelper.getLaborRelStatusOfStaffFilter())
                .and(QFilterPersonHelper.getOtClassifyFilter())
                .and(QFilterCommonHelper.getCurrentVersionFilter())
                .and(QFilterCommonHelper.getDataStatusFilter())
                .and(QFilterCommonHelper.getInitStatusFinishFilter());

        // 合并 filters
        List<QFilter> filterList = new ArrayList<>();
        filterList.add(baseFilter);
        if (filters != null && filters.length > 0) {
            filterList.addAll(Arrays.asList(filters));
        }

        // 防止 selectFields 为空
        if (StringUtils.isBlank(selectFields)) {
            selectFields = "id";
        }
        // 执行查询并返回结果
        return QueryServiceHelper.query(DEP_EMP_ENTITY_ID, selectFields, filterList.toArray(new QFilter[0]));
    }

}