package nckd.jxccl.hr.hstu.business;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.hr.hbp.business.openservicehelper.hrpi.HRPIDepempServiceHelper;
import nckd.jxccl.base.common.constant.FromConstant;
import nckd.jxccl.base.common.utils.ConvertUtil;
import nckd.jxccl.base.common.utils.DateUtil;
import nckd.jxccl.base.org.helper.AdminOrgHelper;
import nckd.jxccl.base.org.helper.PersonHelper;
import nckd.jxccl.base.orm.helper.QFilterCommonHelper;
import nckd.jxccl.hr.hstu.common.HonorStudentConstant;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringJoiner;

/**
 * 【优秀生】-生成评价问卷
 * @author W.Y.C
 * @date 2025/7/2 16:41
 * @version 1.0
 */
public class CreateEvalQuestService {

    private static final Log logger = LogFactory.getLog(CreateEvalQuestService.class);


    /**
     * 批量生成评价问卷
     * @param evalQuestTplId 评价问卷模板Id
     * @param selectedEvalRule 评价规则
     * @param selectedPerson 被评价人
     * @return: void
     * @author W.Y.C
     * @date: 2025/07/02 17:24
     */
    public static void createEvalQuest(Long evalQuestTplId, DynamicObject selectedEvalRule, DynamicObjectCollection selectedPerson) {

        //同级人员取数条件：
        //1.最新一条党政职务履历为副处级及以上人员不作为同级取数
        //2.工作性质大类过滤掉编码为H01（居休）、H02（退养）、H03（随任）、H04（女工长假）、H05（长期工伤病休）、H06（长期病休）、H07（其他不在岗）、J（外派）的员工,不做为同级
        //3.除去领导和上级
        //4.入职超半年、职位非协理员
        logger.info("【职位体系】-生成综合评测问卷-开始");


        StringJoiner ruleColum = new StringJoiner(",");
        //其他同级评分人比例
        ruleColum.add(HonorStudentConstant.NCKD_PROPORTION);
        //同级评分人上限人数
        ruleColum.add(HonorStudentConstant.NCKD_UPPER);
        //同级评分人下限人数
        ruleColum.add(HonorStudentConstant.NCKD_LOWER);
        selectedEvalRule = QueryServiceHelper.queryOne(HonorStudentConstant.EVALUATIONRULE_ENTITYID, ruleColum.toString(), new QFilter[]{QFilterCommonHelper.getIdEqFilter(selectedEvalRule.getPkValue())});
        int proportion = selectedEvalRule.getInt(HonorStudentConstant.NCKD_PROPORTION);
        int upper = selectedEvalRule.getInt(HonorStudentConstant.NCKD_UPPER);
        int lower = selectedEvalRule.getInt(HonorStudentConstant.NCKD_LOWER);
        int otherLower = lower * proportion / 100;
        try{
            // 用工关系状态：正式员工、试用员工、见习期员工
            for (DynamicObject row : selectedPerson) {
                DynamicObject depEmpRow = (DynamicObject)row.getDynamicObject(FromConstant.BASEDATAID_KEY);

                Map<String, Object> depEmp = HRPIDepempServiceHelper.getDepemp(ConvertUtil.toLong(depEmpRow.getPkValue()));
                Long id = ConvertUtil.toLong(depEmp.get("id"));
                //获取当前人员的组织
                Long adminOrgId = ConvertUtil.toLong(depEmp.get("adminorg_id"));
                //获取当前人员的岗位
                Long positionId = ConvertUtil.toLong(depEmp.get("position_id"));

                //获取领导（获取上上级岗位中的人员作为领导）
                DynamicObject parentParentPosition = QueryServiceHelper.queryOne("hbpm_positionhr","parent.parent.id as parentId",new QFilter[]{QFilterCommonHelper.getIdEqFilter(positionId)});
                QFilter positionFilter = new QFilter("position.id", QCP.equals, parentParentPosition.getLong("parentId"));
                DynamicObjectCollection leadColl =  PersonHelper.getDepEmpEffectivePerson("id,person.name",positionFilter);

                //获取上级（获取上级岗位中的人员作为上级）
                DynamicObject parentPosition = QueryServiceHelper.queryOne("hbpm_positionhr","parent.id as parentId",new QFilter[]{QFilterCommonHelper.getIdEqFilter(positionId)});
                positionFilter = new QFilter("position.id", QCP.equals, parentPosition.getLong("parentId"));
                DynamicObjectCollection superiorColl = PersonHelper.getDepEmpEffectivePerson("id,person.name",positionFilter);

                //获取同级人员
                Set<Long> excludePersons = new HashSet<>(leadColl.size()+superiorColl.size());
                excludePersons.add(id);
                leadColl.forEach(dynamicObject -> excludePersons.add(dynamicObject.getLong(HonorStudentConstant.ID_KEY)));
                superiorColl.forEach(dynamicObject -> excludePersons.add(dynamicObject.getLong(HonorStudentConstant.ID_KEY)));
                DynamicObjectCollection peers = getPeers(adminOrgId,excludePersons, proportion,upper, lower, otherLower);


                System.out.println();
            }

        }catch (Exception e){
            logger.error("【职位体系】-生成综合评测问卷-异常",e);
        }
        logger.info("【职位体系】-生成综合评测问卷-结束");
    }

    /**
     * 获取同级人员
     *  1.最新一条党政职务履历为副处级及以上人员不作为同级取数
     *  2.工作性质大类过滤掉编码为H01（居休）、H02（退养）、H03（随任）、H04（女工长假）、H05（长期工伤病休）、H06（长期病休）、H07（其他不在岗）、J（外派）的员工,不做为同级
     *  3.除去领导和上级
     *  4.入职超半年、职位非协理员
     * @param adminOrgId 当前组织ID
     * @param excludePersons 排除的ID
     * @return: kd.bos.dataentity.entity.DynamicObjectCollection
     * @author W.Y.C
     * @date: 2025/07/08 17:48
     */
    private static DynamicObjectCollection getPeers(Long adminOrgId,Set<Long> excludePersons,int proportion,int upper,int lower,int otherLower){
        /*
        同级人数取数逻辑
        │
        ├─ 读取评价规则：proportion（其他同级评分人比例）, upper（同级评分人上限人数）, lower（同级评分人下限人数） → 计算 otherlower
        │
        ├─ 1：查询当前组织内的同级员工（排除上级、自己）
        │   ├─ 1.1：若人数 >= lower → 只取前 upper 个
        │   └─ 1.2：否则 继续向上级组织查找（最多 10 层）
        │
        ├─ 2：查询同级其他部门在职人员
        │
        └─ 返回最终的同级评分人列表
        */
        //最终的结果
        DynamicObjectCollection finalResult = new DynamicObjectCollection();

        //------ 基本条件 begin------
        QFilter baseFilter = new QFilter(HonorStudentConstant.ID_KEY, QCP.not_in, excludePersons);
        //入职超半年
        baseFilter.and(new QFilter("startdate",QCP.less_equals, DateUtil.minusMonths(DateUtil.now(),6)));
        //职位非协理员
        baseFilter.and(new QFilter("position.name", QCP.not_like, "%协理员%"));
        //------ 基本条件 end------

        //1.查询当前组织内的同级员工
        QFilter adminOrgFilter = new QFilter("adminorg.id", QCP.equals, adminOrgId);
        DynamicObjectCollection depEmpEffectivePerson = PersonHelper.getDepEmpEffectivePerson("id,person.name", baseFilter, adminOrgFilter);

        //当前组织人数小于下限人数，继续向上级组织查找直到 >= lower
        if(depEmpEffectivePerson.size() < lower){
            recursionGetParentPeers(adminOrgId, depEmpEffectivePerson, baseFilter, lower, 1);
        }
        //截取前 upper 个同级人员
        Collections.shuffle(depEmpEffectivePerson);
        List<DynamicObject> dynamicObjects = depEmpEffectivePerson.subList(0, depEmpEffectivePerson.size() > upper ? upper-1 : depEmpEffectivePerson.size() -1);
        finalResult.addAll(dynamicObjects);

        //2.查询同级其他部门在职人员

        return finalResult;
    }

    /**
     * 递归上级组织获取人员
     * @param adminOrgId 组织id
     * @param persons 已获取的同级人员
     * @param baseFilter 基本条件
     * @param lower 同级评分人下限人数
     * @param count 递归累计
     * @return: void
     * @author W.Y.C
     * @date: 2025/07/09 16:03
     */
    private static void recursionGetParentPeers(Long adminOrgId,DynamicObjectCollection persons,QFilter baseFilter,int lower,Integer count){
        Long parentOrg = AdminOrgHelper.getParentOrg(adminOrgId);
        if(parentOrg == null){
            return;
        }
        QFilter adminOrgFilter = new QFilter("adminorg.id", QCP.equals, parentOrg);
        DynamicObjectCollection depEmpEffectivePerson = PersonHelper.getDepEmpEffectivePerson("id,person.name", baseFilter, adminOrgFilter);
        persons.addAll(depEmpEffectivePerson);
        if(persons.size() >= lower || count > 10){
            return;
        }else{
            count++;
            recursionGetParentPeers(parentOrg,persons,baseFilter,lower,count);
        }
    }
}