package nckd.jxccl.hr.hstu.plugin.form;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.form.CloseCallBack;
import kd.bos.form.ConfirmCallBackListener;
import kd.bos.form.MessageBoxOptions;
import kd.bos.form.MessageBoxResult;
import kd.bos.form.control.Control;
import kd.bos.form.events.ClosedCallBackEvent;
import kd.bos.form.events.MessageBoxClosedEvent;
import kd.bos.form.plugin.AbstractFormPlugin;
import kd.bos.schedule.api.JobInfo;
import kd.bos.schedule.api.JobType;
import kd.bos.schedule.form.JobForm;
import kd.bos.schedule.form.JobFormInfo;
import kd.sdk.plugin.Plugin;
import nckd.jxccl.base.common.utils.ConvertUtil;
import nckd.jxccl.hr.hstu.business.CreateEvalQuestService;
import nckd.jxccl.hr.hstu.common.HonorStudentConstant;
import nckd.jxccl.hr.hstu.task.CreateEvalQuestTask;
import nckd.jxccl.hr.hstu.task.CreateEvalQuestTaskClick;

import java.util.EventObject;
import java.util.HashMap;
import java.util.Map;

/**
* 生成评价对象表单插件
* @author W.Y.C
* @date 2025/7/1 10:19
* @version 1.0
*/
public class CreateEvalQuestFormPlugin extends AbstractFormPlugin implements Plugin {

    @Override
    public void registerListener(EventObject e) {
        this.addClickListeners(HonorStudentConstant.OP_BTN_OK);
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);
        Long evalQuestTplId = ConvertUtil.toLong(this.getView().getFormShowParameter().getCustomParams().get(HonorStudentConstant.NCKD_EVALQUESTTPLID));
        this.getModel().setValue(HonorStudentConstant.NCKD_EVALQUESTTPLID, evalQuestTplId);
    }

    @Override
    public void click(EventObject evt) {
        Control source = (Control) evt.getSource();
        if (HonorStudentConstant.OP_BTN_OK.equals(source.getKey())) {
            DynamicObject selectedEvalRule = (DynamicObject)this.getModel().getValue(HonorStudentConstant.NCKD_SELECTEDEVALRULE);
            if(selectedEvalRule == null){
                this.getView().showTipNotification("请选择评价规则");
                return;
            }
           DynamicObjectCollection selectedPerson = (DynamicObjectCollection)this.getModel().getValue(HonorStudentConstant.NCKD_SELECTEDPERSON);
            if(selectedPerson.isEmpty()){
                this.getView().showTipNotification("请选择评价对象");
                return;
            }

            Map<String, Object> returnMap = new HashMap<>(2);
            returnMap.put("selectedEvalRule", selectedEvalRule);
            returnMap.put("selectedPerson", selectedPerson);
            this.getView().showConfirm("是否确认生成已选人员的评测问卷？", MessageBoxOptions.YesNo, new ConfirmCallBackListener(HonorStudentConstant.SUBMIT_KEY, this));
        }
    }

    @Override
    public void confirmCallBack(MessageBoxClosedEvent messageBoxClosedEvent) {
        String callBackId = messageBoxClosedEvent.getCallBackId();
        if(HonorStudentConstant.SUBMIT_KEY.equals(callBackId) && messageBoxClosedEvent.getResult() == MessageBoxResult.Yes){
            Long evalQuestTplId = ConvertUtil.toLong(this.getView().getFormShowParameter().getCustomParams().get(HonorStudentConstant.NCKD_EVALQUESTTPLID));
            DynamicObject selectedEvalRule = (DynamicObject)this.getModel().getValue(HonorStudentConstant.NCKD_SELECTEDEVALRULE);
            DynamicObjectCollection selectedPerson = (DynamicObjectCollection)this.getModel().getValue(HonorStudentConstant.NCKD_SELECTEDPERSON);

            if(selectedPerson.size() > 30) {
                //选择生成人员超过30人开启异步任务
                // 1. 创建任务实例
                JobInfo jobInfo = new JobInfo();
                jobInfo.setTaskClassname(CreateEvalQuestTask.class.getName());
                jobInfo.setParams(new HashMap<>()); // 可传递参数
                jobInfo.setJobType(JobType.REALTIME);
                jobInfo.setSuccessNotify(Boolean.TRUE);

                // 2. 配置进度界面
                JobFormInfo formInfo = new JobFormInfo(jobInfo);
                formInfo.setCaption("生成综合评测问卷进度");
                formInfo.setCloseCallBack(new CloseCallBack(EvalQuestTplListPlugin.class.getName(), HonorStudentConstant.TASK_CREATEEVAL));
                formInfo.setClickClassName(CreateEvalQuestTaskClick.class.getName());
                // 允许用户中途取消
                //formInfo.setCanStop(true);
                // 允许切到后台
                //formInfo.setCanBackground(true);

                // 3. 发布任务并显示进度界面
                JobForm.dispatch(formInfo, this.getView().getParentView());

                this.getView().close();
            }else{
                CreateEvalQuestService.createEvalQuest(evalQuestTplId, selectedEvalRule, selectedPerson);
            }

        }
    }

    @Override
    public void closedCallBack(ClosedCallBackEvent closedCallBackEvent) {
        System.out.println();
    }
}